# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

from pandas import to_datetime as pddt
from Octeres.SL.location_to_mask_graph import XLocation_to_Mask_None
from Octeres.SL.abstract_location_to_mask import ABC_Location_to_Mask_Graph
from Octeres.SL.obj_type import Obj_Type

_, MACHINE_NAME = __name__.rsplit("ltm_", 1)


class Location_to_Mask(ABC_Location_to_Mask_Graph):
    machine_name = MACHINE_NAME
    bit_total = 32
    names = (
        (Obj_Type.NODE.value, (1, 5, 1), "salae-c{node}"),
        (Obj_Type.CGROUP.value, (0, 8, 1), "salae-c{node}[{cgroup}]"),
    )
    obj_types = [Obj_Type.ROOT.value, Obj_Type.NODE.value, Obj_Type.CGROUP.value]

    def get_parents(self, machine_name: str, node_name: str) -> list:
        if machine_name == node_name:
            parents = []
        else:
            if "[" in node_name and "]" in node_name:
                head, tail = node_name.rsplit("[", 1)
                parents = [head]
            else:
                parents = [machine_name]
        return parents

    def get_obj_type(self, machine_name: str, node_name: str) -> str:
        if machine_name == node_name:
            obj_type = Obj_Type.ROOT.value
        else:
            if "[" in node_name and "]" in node_name:
                obj_type = Obj_Type.CGROUP.value
            else:
                obj_type = Obj_Type.NODE.value
        return obj_type


ltm_lst = [
    {
        "ts": pddt("1970-01-01"),
        "te": pddt("2021-01-01"),
        "ltm_class": XLocation_to_Mask_None,
        "name": MACHINE_NAME,
    },
    {
        "ts": pddt("2021-01-01"),
        "te": pddt("2038-01-01"),
        "ltm_class": Location_to_Mask,
        "pk": 20090,
        "name": MACHINE_NAME,
    },
]
