# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import copy
import functools
import inspect

import numpy as np


def wrap_and_detect_modification(func):
    @functools.wraps(func)
    def decorate(*args, **kwargs) -> dict:
        sig = inspect.signature(func)
        sig_lst = list(sig.parameters.keys())
        backedup_args = copy.deepcopy(args)
        backedup_kwargs = copy.deepcopy(kwargs)
        response = func(*args, **kwargs)
        idx = 0
        for idx, (arg, arg_copy) in enumerate(zip(args, backedup_args)):
            name = sig_lst[idx]
            if name == "self":
                continue
            assert id(arg), id(arg_copy)
            try:
                assert arg == arg_copy, f"{func.__name__}:{idx}:({name})arg {arg} has been modified"
            except ValueError:
                if np and str(type(arg)) in (
                    "<class 'Octeres.bitmask.EnhancedBitmask'>",
                    "<class 'numpy.ndarray'>",
                ):
                    # for numpy support
                    assert np.equal(arg, arg_copy).all(), f"{func.__name__}:{idx}:({name})arg {arg} has been modified"
                else:
                    raise
        last_idx = idx
        for idx, (kwarg, kwarg_copy) in enumerate(zip(kwargs, backedup_kwargs)):
            name = sig_lst[idx + last_idx]
            assert id(kwarg) == id(kwarg_copy)
            try:
                assert kwarg == kwarg_copy, f"{func.__name__}:{idx}:{name}kwarg {kwarg} has been modified"
            except ValueError:
                if np and str(type(kwarg)) in (
                    "<class 'Octeres.bitmask.EnhancedBitmask'>",
                    "<class 'numpy.ndarray'>",
                ):
                    # for numpy support
                    assert np.equal(kwarg, kwarg_copy).all(), f"{func.__name__}:{idx}:{name}kwarg {kwarg} has been modified"
                else:
                    raise
        return response

    return decorate
